import { AxiosInstance, AxiosRequestConfig, AxiosResponse } from 'axios';
import { Cache } from 'cache-manager';
import { ErrorMiddleware, HttpModuleOptions, RequestMiddleware, ResponseMiddleware } from './interfaces/http.interface';
export declare const HTTP_MODULE_OPTIONS = "HTTP_MODULE_OPTIONS";
export declare class HttpService {
    private readonly httpModuleOptions;
    private readonly cacheManager;
    private readonly logger;
    private readonly axiosInstance;
    private readonly requestMiddlewares;
    private readonly responseMiddlewares;
    private readonly errorMiddlewares;
    constructor(httpModuleOptions: HttpModuleOptions, cacheManager: Cache);
    private getAxiosConfig;
    private setupRetry;
    private setupCache;
    private generateCacheKey;
    private setupInterceptors;
    registerRequestMiddleware(middleware: RequestMiddleware): void;
    registerResponseMiddleware(middleware: ResponseMiddleware): void;
    registerErrorMiddleware(middleware: ErrorMiddleware): void;
    request<T = any>(config: AxiosRequestConfig): Promise<AxiosResponse<T>>;
    get<T = any>(url: string, config?: AxiosRequestConfig): Promise<AxiosResponse<T>>;
    post<T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<AxiosResponse<T>>;
    put<T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<AxiosResponse<T>>;
    patch<T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<AxiosResponse<T>>;
    delete<T = any>(url: string, config?: AxiosRequestConfig): Promise<AxiosResponse<T>>;
    head<T = any>(url: string, config?: AxiosRequestConfig): Promise<AxiosResponse<T>>;
    options<T = any>(url: string, config?: AxiosRequestConfig): Promise<AxiosResponse<T>>;
    getAxiosInstance(): AxiosInstance;
}
