"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var HttpService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpService = exports.HTTP_MODULE_OPTIONS = void 0;
const common_1 = require("@nestjs/common");
const axios_1 = require("axios");
const axios_retry_1 = require("axios-retry");
const cache_manager_1 = require("@nestjs/cache-manager");
const http_status_codes_1 = require("http-status-codes");
const qs = require("qs");
exports.HTTP_MODULE_OPTIONS = 'HTTP_MODULE_OPTIONS';
let HttpService = HttpService_1 = class HttpService {
    constructor(httpModuleOptions, cacheManager) {
        this.httpModuleOptions = httpModuleOptions;
        this.cacheManager = cacheManager;
        this.logger = new common_1.Logger(HttpService_1.name);
        this.requestMiddlewares = [];
        this.responseMiddlewares = [];
        this.errorMiddlewares = [];
        this.axiosInstance = axios_1.default.create(this.getAxiosConfig(httpModuleOptions));
        this.setupRetry(httpModuleOptions.retry);
        this.setupCache(httpModuleOptions.cache);
        this.setupInterceptors(httpModuleOptions);
    }
    getAxiosConfig(options) {
        const { retry, cache, interceptors } = options, axiosOptions = __rest(options, ["retry", "cache", "interceptors"]);
        const config = Object.assign({ timeout: options.timeout || 10000, paramsSerializer: (params) => qs.stringify(params, { arrayFormat: 'brackets' }) }, axiosOptions);
        if (options.headers) {
            config.headers = options.headers;
        }
        return config;
    }
    setupRetry(retryOptions) {
        if (!retryOptions)
            return;
        const options = typeof retryOptions === 'boolean' ? {} : retryOptions;
        (0, axios_retry_1.default)(this.axiosInstance, {
            retries: options.retries || 3,
            retryDelay: axios_retry_1.default.exponentialDelay,
            retryCondition: (error) => {
                if (options.retryCondition && typeof options.retryCondition === 'function') {
                    return options.retryCondition(error);
                }
                const statusCodesToRetry = options.statusCodesToRetry || [
                    http_status_codes_1.StatusCodes.REQUEST_TIMEOUT,
                    http_status_codes_1.StatusCodes.TOO_MANY_REQUESTS,
                    http_status_codes_1.StatusCodes.INTERNAL_SERVER_ERROR,
                    http_status_codes_1.StatusCodes.BAD_GATEWAY,
                    http_status_codes_1.StatusCodes.SERVICE_UNAVAILABLE,
                    http_status_codes_1.StatusCodes.GATEWAY_TIMEOUT,
                ];
                const methodsToRetry = options.methodsToRetry || [
                    'get', 'head', 'options', 'delete', 'put',
                ];
                const status = error.response ? error.response.status : null;
                const method = error.config ? error.config.method : null;
                return (axios_retry_1.default.isNetworkError(error) ||
                    (status !== null && statusCodesToRetry.includes(status) &&
                        method !== null && methodsToRetry.includes(method.toLowerCase())));
            },
        });
    }
    setupCache(cacheOptions) {
        if (!cacheOptions || !this.cacheManager)
            return;
        const options = typeof cacheOptions === 'boolean' ? {} : cacheOptions;
        this.axiosInstance.interceptors.request.use(async (config) => {
            var _a;
            const methods = options.methods || ['get'];
            if (!methods.includes(((_a = config.method) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || '')) {
                return config;
            }
            const cacheKey = this.generateCacheKey(config, options);
            const cachedResponse = await this.cacheManager.get(cacheKey);
            if (cachedResponse) {
                this.logger.debug(`Cache hit for ${cacheKey}`);
                config.adapter = (() => Promise.resolve(cachedResponse));
            }
            return config;
        });
        this.axiosInstance.interceptors.response.use(async (response) => {
            var _a;
            const methods = options.methods || ['get'];
            if (!methods.includes(((_a = response.config.method) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || '')) {
                return response;
            }
            if (options.shouldCache && !options.shouldCache(response)) {
                return response;
            }
            const cacheKey = this.generateCacheKey(response.config, options);
            try {
                const safeResponse = {
                    data: response.data,
                    status: response.status,
                    statusText: response.statusText,
                    headers: response.headers,
                    config: {
                        url: response.config.url,
                        method: response.config.method,
                        headers: response.config.headers
                    }
                };
                const ttl = options.ttl || 60000;
                await this.cacheManager.set(cacheKey, safeResponse, ttl);
                this.logger.debug(`Cached response for ${cacheKey} (TTL: ${ttl}ms)`);
            }
            catch (error) {
                this.logger.warn(`Error al almacenar respuesta en caché: ${error.message}`);
            }
            return response;
        });
    }
    generateCacheKey(config, options) {
        var _a;
        if (options.cacheKey) {
            if (typeof options.cacheKey === 'function') {
                return options.cacheKey(config);
            }
            return options.cacheKey;
        }
        const method = ((_a = config.method) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || 'get';
        const url = config.url || '';
        let params = '';
        let data = '';
        try {
            const getCircularReplacer = () => {
                const seen = new WeakSet();
                return (key, value) => {
                    if (typeof value === 'object' && value !== null) {
                        if (seen.has(value)) {
                            return '[Circular Reference]';
                        }
                        seen.add(value);
                    }
                    return value;
                };
            };
            params = config.params ? JSON.stringify(config.params, getCircularReplacer()) : '';
            data = config.data ? JSON.stringify(config.data, getCircularReplacer()) : '';
        }
        catch (error) {
            this.logger.warn(`Error al serializar datos para caché: ${error.message}`);
            params = config.params ? 'params-present' : '';
            data = config.data ? 'data-present' : '';
        }
        return `http-cache:${method}:${url}:${params}:${data}`;
    }
    setupInterceptors(options) {
        var _a, _b;
        if ((_a = options.interceptors) === null || _a === void 0 ? void 0 : _a.request) {
            for (const interceptor of options.interceptors.request) {
                this.axiosInstance.interceptors.request.use(interceptor.onFulfilled, interceptor.onRejected);
            }
        }
        if ((_b = options.interceptors) === null || _b === void 0 ? void 0 : _b.response) {
            for (const interceptor of options.interceptors.response) {
                this.axiosInstance.interceptors.response.use(interceptor.onFulfilled, interceptor.onRejected);
            }
        }
        this.axiosInstance.interceptors.request.use(async (config) => {
            try {
                let currentConfig = Object.assign({}, config);
                const sortedMiddlewares = [...this.requestMiddlewares]
                    .sort((a, b) => (a.priority || 0) - (b.priority || 0));
                for (const middleware of sortedMiddlewares) {
                    currentConfig = await middleware.process(currentConfig);
                }
                return currentConfig;
            }
            catch (error) {
                this.logger.error('Error in request middleware', error);
                return Promise.reject(error);
            }
        });
        this.axiosInstance.interceptors.response.use(async (response) => {
            try {
                let currentResponse = Object.assign({}, response);
                const sortedMiddlewares = [...this.responseMiddlewares]
                    .sort((a, b) => (a.priority || 0) - (b.priority || 0));
                for (const middleware of sortedMiddlewares) {
                    currentResponse = await middleware.process(currentResponse);
                }
                return currentResponse;
            }
            catch (error) {
                this.logger.error('Error in response middleware', error);
                return Promise.reject(error);
            }
        }, async (error) => {
            try {
                let currentError = error;
                const sortedMiddlewares = [...this.errorMiddlewares]
                    .sort((a, b) => (a.priority || 0) - (b.priority || 0));
                for (const middleware of sortedMiddlewares) {
                    currentError = await middleware.process(currentError);
                    if (!(currentError instanceof Error)) {
                        return currentError;
                    }
                }
                return Promise.reject(currentError);
            }
            catch (processError) {
                this.logger.error('Error in error middleware', processError);
                return Promise.reject(processError);
            }
        });
    }
    registerRequestMiddleware(middleware) {
        this.requestMiddlewares.push(middleware);
        this.logger.log(`Registered request middleware with priority ${middleware.priority || 0}`);
    }
    registerResponseMiddleware(middleware) {
        this.responseMiddlewares.push(middleware);
        this.logger.log(`Registered response middleware with priority ${middleware.priority || 0}`);
    }
    registerErrorMiddleware(middleware) {
        this.errorMiddlewares.push(middleware);
        this.logger.log(`Registered error middleware with priority ${middleware.priority || 0}`);
    }
    request(config) {
        return this.axiosInstance.request(config);
    }
    get(url, config) {
        return this.axiosInstance.get(url, config);
    }
    post(url, data, config) {
        return this.axiosInstance.post(url, data, config);
    }
    put(url, data, config) {
        return this.axiosInstance.put(url, data, config);
    }
    patch(url, data, config) {
        return this.axiosInstance.patch(url, data, config);
    }
    delete(url, config) {
        return this.axiosInstance.delete(url, config);
    }
    head(url, config) {
        return this.axiosInstance.head(url, config);
    }
    options(url, config) {
        return this.axiosInstance.options(url, config);
    }
    getAxiosInstance() {
        return this.axiosInstance;
    }
};
exports.HttpService = HttpService;
exports.HttpService = HttpService = HttpService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(exports.HTTP_MODULE_OPTIONS)),
    __param(1, (0, common_1.Optional)()),
    __param(1, (0, common_1.Inject)(cache_manager_1.CACHE_MANAGER)),
    __metadata("design:paramtypes", [Object, Object])
], HttpService);
//# sourceMappingURL=http.service.js.map