import { ModuleMetadata, Type } from '@nestjs/common';
import { AxiosRequestConfig } from 'axios';
export declare enum CacheStrategy {
    CACHE_FIRST = "cache-first",
    SIDE_CACHE = "side-cache",
    ONLY_CACHE = "only-cache",
    NO_CACHE = "no-cache"
}
export interface RedisConfig {
    host?: string;
    port?: number;
    password?: string;
    db?: number;
}
export interface CacheRouteConfig {
    pattern: string | RegExp;
    strategy: CacheStrategy;
    ttl?: number;
    methods?: string[];
}
export interface CacheOptions {
    ttl?: number;
    max?: number;
    methods?: string[];
    strategy?: CacheStrategy;
    redis?: RedisConfig;
    routes?: CacheRouteConfig[];
    shouldCache?: (response: any) => boolean;
    cacheKey?: string | ((config: any) => string);
}
export interface RetryOptions {
    retries?: number;
    retryDelay?: number;
    statusCodesToRetry?: number[];
    methodsToRetry?: string[];
    retryCondition?: (error: any) => boolean;
}
export interface RequestInterceptor {
    onFulfilled?: (config: any) => any | Promise<any>;
    onRejected?: (error: any) => any;
}
export interface ResponseInterceptor {
    onFulfilled?: (response: any) => any;
    onRejected?: (error: any) => any;
}
export interface InterceptorsConfig {
    request?: RequestInterceptor[];
    response?: ResponseInterceptor[];
}
export interface HttpModuleOptions extends AxiosRequestConfig {
    retry?: RetryOptions;
    cache?: CacheOptions;
    interceptors?: InterceptorsConfig;
}
export interface HttpOptionsFactory {
    createHttpOptions(): Promise<HttpModuleOptions> | HttpModuleOptions;
}
export interface HttpModuleAsyncOptions extends Pick<ModuleMetadata, 'imports'> {
    useExisting?: Type<HttpOptionsFactory>;
    useClass?: Type<HttpOptionsFactory>;
    useFactory?: (...args: any[]) => Promise<HttpModuleOptions> | HttpModuleOptions;
    inject?: any[];
}
export interface RequestMiddleware {
    priority?: number;
    process(config: any): Promise<any> | any;
}
export interface ResponseMiddleware {
    priority?: number;
    process(response: any): Promise<any> | any;
}
export interface ErrorMiddleware {
    priority?: number;
    process(error: any): Promise<any> | any;
}
